{*******************************************************
* ViewPort Conduit                                v2.1 *
* Share Propeller Memory with the ViewPort Application *
* (C) 2008 myDanceBot.com, Inc.                        *
********************************************************}{{
Use this Object to share memory with the ViewPort Application during development.
Once memory is shared, the value of a variable can be viewed and changed while the
program is running.  Data from other objects can be displayed and their behavior
controlled from ViewPort's graphical interface.

Features:
-Transfers data at 2Mbps Full-Duplex over a Parallax USB-Serial connection or
 115Kbps over other Serial connections.
-Manages configuration of variables and registration of ViewPort objects
-Automatically switches between 2Mbps and 115Kbps depending on ViewPort setting

Use:
-Include this object in your program:
        Obj vp:"ViewPortConduit"
-Register and initialize other components:
        vp.register(component,component configuration string)
        *See component documentation for details on how to initialize them
-Share memory with ViewPort
        vp.share(@varA,@varC)
        *All memory between and including varA and varC will be shared with Viewport
-Start the Conduit
        vp.start()
-Optionally configure ViewPort's Interface
        vp.channels(string("a,b,c"))
        *Assign names to the longs being shared

        vp.view(string("a,b"))
        *Graph these variables

        vp.edit(string("a=True,b=10,c=5(1..10)"))
        *Set up Window editing controls for variables a,b,c:
        *a will be set up with a checkbox- True=selected=1, False=unselected=0
        *b will be set up with a textbox starting at 10, allowing entry of numbers,
                %binary number, and $hexadecimal numbers
        *c will be set up with a scrollbar, default at 5, sliding from 1..10

        vp.trigger(string("ina[16]+,a>50,b[rfx01]"))
        *ina will trigger when bit 16 rises
        *a will trigger when its value rises above 50
        *b will trigger when bit 0 is 1, bit 2 is 0, bit 3 is falling, bit 4 is rising

        vp.decode(string("adc=ina[0-7]"))
        *adc will be decoded as an analog value made up of bits 0..7 of ina

        vp.general(string("timescale=1ms/div,title=analog io,mode=dso"))
        *sets timescale to 1ms/div
        *sets title of graph to "analog io"
        *sets ViewPort mode to "dso", valid are: dso,lsa,video,fuzzy,visualizer}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  TXPIN=|<30  
  RXPIN=|<31
PUB share(firstVarPtr,lastVarPtr)
  if lastVarPtr
    varLen:=(lastVarPtr-firstVarPtr+4)/4
    varPtr:=firstVarPtr
    txvar~
  resyncP:=1200/4/(varlen+tvlen+qslen+fzlen)
  if resyncP<2
    resyncP:=1
  resyncCnt:=resyncP
  return cognew(@entry, varPtr)
pub config(sptr)
  if not txstrPtr0
    txStrPtr0:=sptr
  elseif not txstrPtr1
    txStrPtr1:=sptr
  elseif not txstrPtr2
    txStrPtr2:=sptr
  elseif not txstrPtr3
    txStrPtr3:=sptr
  elseif not txstrPtr4
    txStrPtr4:=sptr
  elseif not txstrPtr5
    txStrPtr5:=sptr          
pub register(mPtr)|mode,cfg
    'mptr points to memory, low 2 bits tell what mode
    'data is read/written there by txdata,rxdata
    '0th entry contains config for vp, like size,ncog,qscog
  mode:=mptr & 3 'get low 2 bits, tv, freeprop,qs,fuzz
  mPtr-=mode
  cfg:=long[mPtr]
  if mode==0
    case cfg
      1..2:tvLen:=6000
      3:tvLen:=6000+1
      4:tvLen:=3000+1
      OTHER: tvLen:=1500
    tvPtr:=mptr
    txtv~
  elseif mode==1'     (config>>16)==0
 'freeprop
 'freeprop is special, length is updated in first item
 'not considered a component, separate memptr stores mem
 '    t1:=config '@packetsize
     memPtr:=mPtr+8
  elseif mode==2
    'quicksample
    qsPtr:=mptr
    ncog:=cfg & 1
    qscog:=cfg-ncog+1
    ncog:=ncog*3+1   '1 or 4
 '   modeN:=ncog
    sync:=mPtr+16
    xtime:=sync+4
    qsLen:=ncog*360+6
    txqs~
  else
    'fuzz
    fzPtr:=mPtr
    fzLen:=cfg/4
    txfz~
DAT
{************************************
* Assembly language conduit driver  *
*************************************}

                        org
entry 'read in data
                        sub     tximask,#1
                        sub     tximask,txmask
                        mov     txcnt,hsec
                        add     txcnt,cnt
                        'wait until things settle
                        waitcnt txcnt,#100
                        or      dira,txmask 'dira[tx]=1
                        or      outa,txmask 'outa[tx]=1, startbit pulls low
                        jmp     #txcfgstr
 'receive an instruction: x,y   jumped from transmit got a low signal, either startbit or 0's                     
receive                 or      outa,txmask 'set tx high- done sending
                        mov     txcnt,cnt  'measure low time
                        waitpeq rxmask,rxmask 'wait until stopbit,rx=1
                        mov     t1,cnt 'measure beginning of stop bit
                        sub     txcnt,t1
                        neg     txcnt,txcnt  't1-txcnt , -(-t1+txcnt)
                        cmp     txcnt,stopticks wc 
            if_c        jmp     #txErr 'must receive >2 low bits                                               
                        waitpne rxmask,rxmask 'wait until starbit rx=0                        
 'now at begin of startbit, so wait .5 till get start+data             
                        mov     rxcnt,cnt 'measure end of stopbit
                        mov     t3,rxcnt't2-t3
                        sub     t3,t1   't2-t3t2 holds length of stopbit
                        't2-t1=stop, t2+ (-t1)
{{t2 measures stopbit length.
  if t2<80, 80mhz/2mbit=40cycles
  stopbits is 2x bitticks
  if you're running at a different frequency, change bitticks, stopticks and xtf
  for example, if you're running at 96mhz, bitticks should be=48, stopticks=96}}
                        cmp     t3,#60 wc     't2-t3c set if stopbit < 80
                        mov     t2,#19        'read 2 bytes into t1, skip first startbit
                                                
            if_c        mov     bitticks,#40
            if_c        mov     stopticks,#80
            if_c        mov     xt,xt2mb
            if_c        add     rxcnt,#48
            if_c        jmp     #bit
                        cmp     t3,#120 wc     'c set if stepbit<120
            if_c        mov     bitticks,#80
            if_c        mov     stopticks,#160
            if_c        mov     xt,xt1mb
            if_c        add     rxcnt,#88 '108-20
            if_c        jmp     #bit                         
                        mov     bitticks,bitticks115k
                        mov     stopticks,stopticks115k
                        mov     xt,xt115k
                        add     rxcnt,rxticks115k
                        jmp     #bit
{{constants for slow serial connection- at 80mhz equates to 115kbps.
xt is the time in cycles to send 360 longs, at 96mhz should be 17_280_000}}
rxticks115k            long    1000 
bitticks115k            long    694
stopticks115k           long    1388           
xt2mb                   long    360*2000*1    'time to xmit samplespcog
xt1mb                   long    360*2000*2  
xt115k                  long    360*2000*20 



bit                     waitcnt rxcnt,bitticks
                        test    rxmask,ina      wc
                        rcr     t1,#1     'bit 32 has lsb
                        djnz    t2,#bit  '19 rcr later, bit 32-19=13 has lsb
                        'read in another 3 bytes into t4 just in case
                        mov     t3,#30 
:bit3                   waitcnt rxcnt,bitticks
                        test    rxmask,ina      wc
                        rcr     t4,#1
                        djnz    t3,#:bit3
'process t1,t2                        
                        mov     t2,t1
                        shr     t2,#32-19'19
                        and     t2,#$FF
                        cmp     t2,#254         wz,wc'docontrol on 254
       if_nz_and_nc     jmp     #txErr         '255->txerr
                        shr     t1,#32-9'9
                        and     t1,#$FF
                        cmp     t1,#6           wc
       if_nc_and_z      jmp     #txErr          '254,>5 ->txerr 
              if_z      add     t1,#:docontrol
              if_z      jmp     t1
      
              'write back to var memory
        '0,bp,val -> varptr(pb*4)=val
        '0,255,val -> cmd(val)
        '0,254,val -> not used
        '0,250.1.2.3,val   -> qs(0.4.8.12)=val
        '0,100..249,val -> fuzz(bp)=val
        '0-99,100-249,250..253,254
                        cmp     t2,#250         wc '>=250 ->qsptr(n)
              if_nc     sub     t2,#250
              if_nc     shl     t2,#2
              if_nc     add     t2,qsPtr
              if_nc     jmp     #:makeInt
                        cmp     t2,#100         wc '>=100 ->fzptr(n)
              if_nc     sub     t2,#100
              if_nc     shl     t2,#2
              if_nc     add     t2,fzPtr
              if_nc     jmp     #:makeInt
                        shl     t2,#2
                        add     t2,varPtr
 'if x<255 then varPtr(x)=y                 
 't1 has lsnibble set, need 3 more
 't2 has destination
 't4 hold last 3 bytes
 :makeInt               mov     t3,t4
                        shr     t3,#23
                        and     t3,#$FF
                        shl     t3,#24
 't3 holds byte #4 shifted left 24
                        or      t1,t3
                        
                        mov     t3,t4
                        shr     t3,#13
                        and     t3,#$FF
                        shl     t3,#16
 't3 holds byte #3 shifted left 16
                        or      t1,t3
                        
                        mov     t3,t4
                        shr     t3,#3
                        and     t3,#$FF
                        shl     t3,#8
 't3 holds byte #2 shifted left 8
                        or      t1,t3
                        wrlong  t1,t2
                        jmp     #txData 'end of receive
 'processes control commands: 255, y                        
:docontrol              jmp     #txErr    '0 not used                            
                        jmp     #txMem    '1: send memory
                        jmp     #txData   '2: txdata  
                        jmp     #restart  '3: restart qs cogs
                        jmp     #txcfgStr '4: cfg
                        mov     t1,qscog  '5: stop qs cogs
                        and     t1,#7     't1=lowest 3 bits of qscog=ids
                        mov     t2,ncog                        
 stopC                  cogstop t1
                        add     t1,#1
                        djnz    t2,#stopC
                        jmp     #txData
  restart               mov     t1,cnt
                        wrlong  t1,sync
                        wrlong  xt,xtime
                        mov     t1,qscog    't1=low 3 bits=id of cog to start,other=address
                        mov     t2,ncog
  startC                coginit t1 nr
                        add     t1,#1
                        djnz    t2,#startC
txData
                        mov     conduitstate,conduitstate2
                        mov     txcnt,cnt
                        add     txcnt,stopticks
                        mov     rxcnt,#1
                        call    #txhdr
txD
txMultB
         txvar          jmp     #txqs 'overwritten with nop if used  
                        mov     t2,varLen
                        mov     t3,varPtr
                        call    #txlongs
         txqs           jmp     #txtv                              
                        mov     t2,qsLen
                        mov     t3,qsPtr
                        call    #txlongs
         txtv           jmp     #txfz                              
                        mov     t2,tvLen
                        mov     t3,tvPtr
                        call    #txlongs                        
         txfz           jmp     #txDDone 'worst is txvar and txfz on
                        mov     t2,fzLen 'so moved tx inline
                        mov     t3,fzPtr
                        call    #txlongs
                        
txDDone                 
txDataCon
'                        jmp     #txMultB                       
                        djnz    resynccnt,#txMultB
                        mov     resynccnt,resyncP
                        call    #transmit 'finish txlongs
                        mov     rxcnt,#2
                        call    #txhdr
                        jmp     #txMultB
                    
txlongs                 call    #transmit
                        rdlong  txByte,t3
                        call    #transmit
                        cmp     txByte,s1 wz
                        add     t3,#4                                
                        call    #transmit
                        call    #transmit
              if_z      call    #translong 'add extra 4bytes                        
'after third byte, escape, by addint 5th byte                        
              if_z      add     txByte,#6                        
                        djnz    t2,#txlongs
txlongs_ret             ret

                      
txErr                   mov     txcnt,cnt
                        add     txcnt,stopticks
                        mov     rxcnt,#5
                        call    #txhdr
                        jmp     #txcfgstr
                        
txhdr                   mov     txByte,s1
                        call    #translong
                        add     txByte,rxcnt
                        ror     txByte,#8
                        call    #translong
txhdr_ret               ret
             
 'sends out memory, header
txMem                   mov     txcnt,cnt
                        add     txcnt,stopticks
    :txloop             mov     t2,memPtr
                        rdlong  t3,t2 'mem[0]=mcnt
                        call    #translong
                        mov     rxcnt,#3
                        call    #txhdr
                        jmp     #:txloop                                                
 'sends out config string, header,varlen1,qslen4,tvlen4,fzlen1,cfg strings                        
txCfgStr                mov     txcnt,cnt
                        add     txcnt,stopticks                        
    :txloop             mov     txByte,varLen
                        add     txByte,conduitState
                        call    #transLong
                        mov     txByte,qsLen 
                        call    #transLong
                        mov     txByte,tvLen 
                        call    #transLong
                        mov     txByte,fzLen 
                        call    #transLong
                        mov     txByte,memPtr 
                        call    #transLong
                        mov     t2,txstrptr0
                        call    #txStr
                        mov     t2,txstrptr1
                        call    #txStr
                        mov     t2,txstrptr2
                        call    #txStr
                        mov     t2,txstrptr3
                        call    #txStr
                        mov     t2,txstrptr4
                        call    #txStr
                        mov     t2,txstrptr5
                        call    #txStr
                        mov     rxcnt,#4
                        call    #txhdr
                        jmp     #:txloop                        
txStr                   add     txcnt,bitticks
                        tjnz    t2,#:chars
                        mov     txByte,#0
                        call    #transmit
                        jmp     #txStr_ret
:chars                  rdbyte  txByte,t2
                        mov     t3,txByte 'txbyte gets clobbered
                        call    #transmit
                        add     t2,#1
                        tjnz    t3,#:chars 'strings ended with 0
txStr_ret               ret                        
 'sends out txbyte as a long
transLong
                        call #transmit
                        call    #transmit
                        call    #transmit
                        call    #transmit
transLong_ret           ret
 'sends out a txbyte
transmit                waitcnt txcnt,bitticks   
                        and     outa,tximask    
                        mov     t1,#8
:bit                    test    rxmask,ina      wc                        
            if_nc       jmp     #receive 'receiving a 0
                        ror     txByte,#1       wc
                        waitcnt txcnt,bitticks     
                        muxc    outa,txmask
                        djnz    t1,#:bit
                        waitcnt txcnt,stopticks                                          
                        or      outa,txmask     
transmit_ret            ret
bitticks                long    40
stopticks               long    80
s1                      long    $E1E1E1E1
txstrptr0               long    0
txstrptr1               long    0
txstrptr2               long    0
txstrptr3               long    0
txstrptr4               long    0
txstrptr5               long    0
 'ptrs to mem to be sent
varPtr                  long    0
qsPtr                   long    0
tvPtr                   long    0
fzPtr                   long    0
memPtr                  long    0

varLen                  long    0
qsLen                   long    0
tvLen                   long    0
fzLen                   long    0
conduitState            long    100<<16
conduitState2           long    101<<16
 'memLen stored in memPtr[0]

modeN                   long    0
 'bit timings, 40/80 for 2mb, 80,160 for 1mb, xt *1 for 2mb, *2 for 1mb
 
twosec                  long    160_000_000
hsec                    long    8_000_000 'time to wait before starting xmit- 80m/10=8m=.1s
resyncp                 long    0
resynccnt               long    0

 'config for qs          
qscog                   long    0
sync                    long    0
xtime                   long    0
ncog                    long    0
 'vars
t1                      long    0 
t2                      long    0
t3                      long    0
t4                      long    0
rxmask                  long    RXPIN
txmask                  long    TXPIN
tximask                 long    0
xt                      res     1
rxcnt                   res     1
txByte                  res     1
txcnt                   res     1